/**
 * First things first.  Comments in C++ are exactly the same as java.
 * For multiple line comments, use the standard slash-start combination.
 * For single line comments use the double-slash combination (//).
 **/
 

/**
 * see my explination of #ifndef and #define in Rational.cc
 */
#ifndef __RATIONAL_H__
#define __RATIONAL_H__


/**
 * Most of my comments are in the implementation file Rational.cc
 * In C++ files are usually split up into header (or interface) files
 * and implementation files.  The header file gives the programming interface
 * of the class and decouples the implementation logic from its signature.
 */
#include <iostream.h>
#include "DivideByZeroException.h"

class Rational {

	// private, public, and protected fields are labels.  Everything
	// after the label, but before another label, has the visibility of 
	// that label.  hence _numerator and _denominator are private, the 
	// constructors etc. public, and the init and normalize functions 
	// again private
	private:
		int _numerator;
		int _denominator;

	public:

		// constructors		
		Rational();
		Rational(int numerator);
		Rational(int numerator, int denominator);
	
		// this is the copy constructor 
		Rational(const Rational& r);
		
		
		// operator overloading is decribed in chapter 11 of
		// the 3rd edition of stroustrup.  We use anytime you
		// see the 'operator' keyword, we're performing some
		// sort of operator overloading
		
		// The assignment operator	
		Rational& operator=(const Rational& r);
		
		Rational power(int power) const;
		
		// this is the pre-increment operator
		const Rational& operator++();
		
		// this is the post-increment operator
		const Rational operator++(int);
		
		// this is the pre-decrement operator
		const Rational& operator--();
		
		// this is the post-decrement operator
		const Rational operator--(int);
			
		// Like java, "==" is the equivalency operator.  That is
		// it tells whether two Rationals are equal.  Remember that
		// "=" is the assignment operator.
		const bool operator==(const Rational& r) const;
		
		const Rational operator-(const Rational& r) const;
		const Rational operator+(const Rational& r) const;
		const Rational operator*(const Rational& r) const;

		// By default in C++, all functions can throw any exception.
		// When you declare that a function throws a particular exception you
		// are restricting the Exceptions it can throw.  This is done mostly
		// for back-compatability
		const Rational operator/(const Rational& r) const
			throw (DivideByZeroException);

		const bool operator<(const Rational& r) const;
		const bool operator<=(const Rational& r) const;
		const bool operator>(const Rational& r) const;
		const bool operator>=(const Rational& r) const;
		
		// This is the output stream operator.  It is used to output
		// rational numbers in a user-specific way.
		friend ostream& operator<<(ostream& s, const Rational& r);	
		
	private:
		
		// We don't want the outside world using these.
		void init(int numerator, int denominator);
		void normalize();
		
};  // in C++, classes end with a semi-colon




#endif // __RATIONAL_H__